#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import shutil
import os
import time
import tarfile
import logging
import subprocess

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-client.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

ACTOR="<mythbackup> "

class Restore():

    def restore_job(self, Location, RestoreDB):
        logger.info(ACTOR+'Starting restore job')
        USER=os.getenv("HOME")
        ## Set temp directory and status file
        TMPDIR="/tmp/mythbuntu-bare/"
        STATUSFILE="/tmp/mythbuntu-bare-status"
        ##Remove temp DB dir and statusfile first in case it already exists
        if os.path.exists(STATUSFILE):
            os.remove(STATUSFILE)
        if os.path.exists(TMPDIR):
            shutil.rmtree(TMPDIR)
        ## Open and Extract tarfile
        os.system("echo '30\nRestoring configuration files'>"+STATUSFILE)
        logger.info(ACTOR+'Restoring files from '+str(Location))
        TF=tarfile.open(Location,mode='r')
        TF.extractall(path="/", members=None)
        TF.close()
        ## Update mythtv db password with backed up password
        logger.info(ACTOR+'Reconfiguring mythtv-database')
        os.system("DEBIAN_FRONTEND=noninteractive dpkg-reconfigure mythtv-database")
        logger.info(ACTOR+'Detecting Master Backend')
        if os.path.isdir("/usr/share/doc/mythtv-backend-master"):
            logger.info(ACTOR+'Master Backend: True')
            if RestoreDB == True or RestoreDB == "True" or RestoreDB == 1:
                logger.info(ACTOR+'Restoring Database: True')
                os.system("echo '40\nRestoring database (this could take a few minutes)'>"+STATUSFILE)
                ##TODO
                restorescript="/usr/share/mythtv/mythconverg_restore.pl"
                pipe = subprocess.Popen([restorescript, '--drop_database', '--create_database', '--directory', TMPDIR], stdout=subprocess.PIPE).communicate()
            else:
                logger.info(ACTOR+'Restoring Database: False')
        else:
            logger.info(ACTOR+'Master Backend: False')
        ## Remove Temp directory and status file
        os.system("echo '90\nCleaning up temp files'>"+STATUSFILE)
        time.sleep(1)
        logger.info(ACTOR+'Cleaning up temp files')
        if os.path.exists(TMPDIR):
            shutil.rmtree(TMPDIR)
        if os.path.exists(STATUSFILE):
            os.remove(STATUSFILE)
        logger.info(ACTOR+'Restore job finished')



#!/usr/bin/python3
 
import sys
from gobject import MainLoop
from dbus.mainloop.glib import DBusGMainLoop
from ubuntuone.platform.credentials import CredentialsManagementTool
import ubuntuone.couch.auth as auth
import urllib.request, urllib.parse, urllib.error
import json
import mimetypes

class U1Files():

    def login(self):
        # Login
        global _login_success
        _login_success = False
       
        DBusGMainLoop(set_as_default=True)
        loop = MainLoop()
       
        def quit(result):
            global _login_success
            loop.quit()
            if result:
                _login_success = True
       
        cd = CredentialsManagementTool()
        d = cd.login()
        d.addCallbacks(quit)
        loop.run()
        if not _login_success:
            sys.exit(1)
        return _login_success
     
    def logout(self):
        # Logout
        DBusGMainLoop(set_as_default=True)
        loop = MainLoop()
       
        def quit(result):
            loop.quit()
       
        cd = CredentialsManagementTool()
        d = cd.clear_credentials()
        d.addCallbacks(quit)
        loop.run()
     
    def create_volume(self, path):
        # Create directory
        base = "https://one.ubuntu.com/api/file_storage/v1/volumes/~/"
        url = base + urllib.parse.quote(path)
        auth.request(url, http_method="PUT")
        answer = auth.request(url)
        status = int(answer[0].get('status'))
        return status

    def put(self, local, remote):
        # Create remote path (which contains volume path)
        base = "https://one.ubuntu.com/api/file_storage/v1/~/"
        answer = auth.request(base + urllib.parse.quote(remote),
                              http_method="PUT",
                              request_body='{"kind":"file"}')
        node = json.loads(answer[1])
        
        # Read info about local file
        data = bytearray(open(local, 'rb').read())
        size = len(data)
        content_type = mimetypes.guess_type(local)[0]
        content_type = content_type or 'application/octet-stream'
        headers = {"Content-Length": str(size),
                   "Content-Type": content_type}
       
        # Upload content of local file to content_path from original response
        base = "https://files.one.ubuntu.com"
        url = base + urllib.parse.quote(node.get('content_path'), safe="/~")
        auth.request(url, http_method="PUT",
                     headers=headers, request_body=data)

        sha1, size, status = self.query(remote)
        return status
     
    def get(self, remote, local):
        # Request metadata
        base = "https://one.ubuntu.com/api/file_storage/v1/~/"
        answer = auth.request(base + urllib.parse.quote(remote))
        node = json.loads(answer[1])
        status = int(answer[0].get('status'))
        
        # Request content
        base = "https://files.one.ubuntu.com"
        url = base + urllib.parse.quote(node.get('content_path'), safe="/~")
        answer = auth.request(url)
        f = open(local, 'wb')
        f.write(answer[1])
        return status

    def get_children(self, path):
        # Request children metadata
        base = "https://one.ubuntu.com/api/file_storage/v1/~/"
        url = base + urllib.parse.quote(path) + "?include_children=true"
        answer = auth.request(url)
        status = int(answer[0].get('status'))
        # Create file list out of json data
        filelist = []
        node = json.loads(answer[1])
        if node.get('has_children') == True:
            for child in node.get('children'):
                child_path = urllib.parse.unquote(child.get('path')).lstrip('/')
                filelist += [child_path]
        return filelist, status
     
    def query(self, path):
        # Request metadata
        base = "https://one.ubuntu.com/api/file_storage/v1/~/"
        url = base + urllib.parse.quote(path)
        answer = auth.request(url)
        node = json.loads(answer[1])
        status = int(answer[0].get('status'))
        # Print interesting info
        return node.get('hash'), node.get('size'), status
     
    def delete(self, path):
        # Delete file
        base = "https://one.ubuntu.com/api/file_storage/v1/~/"
        url = base + urllib.parse.quote(path)
        answer = auth.request(url)
        status = int(answer[0].get('status'))
        auth.request(url, http_method="DELETE")
        return status

if __name__ == "__main__":
    f = U1Files()

    if len(sys.argv) <= 1:
        print("Need more arguments")
        sys.exit(1)

    if sys.argv[1] == "delete":
        f.login()
        f.delete(sys.argv[2])

    if sys.argv[1] == "login":
        f.login()

    if sys.argv[1] == "get":
        f.login()
        print(f.get(sys.argv[2], sys.argv[3]))

    if sys.argv[1] == "list":
        f.login()
        print(f.get_children(sys.argv[2]))

    if sys.argv[1] == "put":
        f.login()
        print(f.put(sys.argv[2], sys.argv[3]))

    if sys.argv[1] == "query":
        f.login()
        print(f.query(sys.argv[2]))

    if sys.argv[1] == "create-volume":
        f.login()
        print(f.create_volume(sys.argv[2]))

    if sys.argv[1] == "logout":
        f.logout()



## -*- coding: utf-8 -*-
#
# «Bare Console» - The serverside configuration/status console for Mythbuntu Bare
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import time
from threading import Thread
import configparser
import sys
import subprocess
import gobject
import socket
import os
import hashlib

CONFIGFILE = "/var/lib/mythtv/bare/mythbuntu-bare.conf"

class BareConsolePlugin(MCCPlugin):
    """Server portion of Mythbuntu Backup and Restore utility"""
    #
    #Load GUI & Calculate Changes
    #

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Bare Console"
        information["icon"] = "gtk-stop"
        information["ui"] = "tab_bare_console"
        MCCPlugin.__init__(self,information)

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        self.changes = {}
        self.status_host_buffer = self.status_text_host.get_buffer()
        self.status_lastbackup_buffer = self.status_text_lastbackup.get_buffer()
        self.status_lastseen_buffer = self.status_text_lastseen.get_buffer()
        self.status_status_buffer = self.status_text_status.get_buffer()
        self.config = configparser.ConfigParser()
        if os.path.exists(CONFIGFILE):
          self.config.read(CONFIGFILE)
          try:
            self.changes['serverip'] = self.config.get("General", "serverip")
          except:
            print("Cannot read serverip, setting default")
            self.changes['serverip'] = subprocess.getoutput("/sbin/ifconfig").split("\n")[1].split()[1][5:]
          try:
            self.changes['managed'] = self.config.get("General", "managed")
          except:
            print("Cannot read managed state, setting default")
            self.changes['managed'] = "False"
          try:
            self.changes['revision'] = self.config.get("General", "revision")
          except:
            print("Cannot read revision, setting default")
            self.changes['revision'] = "0"
          try:
            self.changes['BackupLocation'] = self.config.get("Backup", "storagedir")
          except:
            print("Cannot read Backup Location, setting default")
            self.changes['BackupLocation'] = "/var/lib/mythtv/bare"
          try:
            self.changes['backup_db'] = self.config.get("Backup", "db")
          except:
            print("Cannot read db backup setting, setting default")
            self.changes['backup_db'] = "1"
          try:
            self.changes['backup_schedule'] = self.config.get("Backup", "schedule")
            self.changes['backup_schedule_day'] = self.config.get("Backup", "day")
            self.changes['backup_schedule_hour'] = self.config.get("Backup", "hour")
            self.changes['backup_schedule_minute'] = self.config.get("Backup", "minute")
            self.changes['backup_schedule_weekday'] = self.config.get("Backup", "weekday")
          except:
            print("Cannot read schedule, setting default")
            self.changes['backup_schedule'] = "disabled"
            self.changes['backup_schedule_day'] = "*"
            self.changes['backup_schedule_hour'] = "0"
            self.changes['backup_schedule_minute'] = "00"
            self.changes['backup_schedule_weekday'] = "*"
          try:
            self.changes['serverstoragedir'] = self.config.get("General", "serverstoragedir")
          except:
            print("Cannot read server storage directory, setting default")
            self.changes['serverstoragedir'] = "/var/lib/mythtv/bare/"
          try:
            self.changes['serverport'] = self.config.get("General", "serverport")
          except:
            print("Cannot read server port, setting default")
            self.changes['serverport'] = self.PickUnusedPort()
        else:
          print("Error: Config file not found. Setting defaults")
          self.changes['serverip'] = subprocess.getoutput("/sbin/ifconfig").split("\n")[1].split()[1][5:]
          self.changes['managed'] = "False"
          self.changes['revision'] = "0"
          self.changes['BackupLocation'] = "/var/lib/mythtv/bare"
          self.changes['backup_db'] = "1"
          self.changes['backup_schedule'] = "daily"
          self.changes['backup_schedule_day'] = "*"
          self.changes['backup_schedule_hour'] = "0"
          self.changes['backup_schedule_minute'] = "00"
          self.changes['backup_schedule_weekday'] = "*"
          self.changes['serverstoragedir'] = "/var/lib/mythtv/bare/"
          self.changes['serverport'] = self.PickUnusedPort()

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.hide_stuff()
        if self.changes['managed'] == "True":
          self.managed_radiobutton.set_active(True)
        else:
          self.unmanaged_radiobutton.set_active(True)
        self.serverip_entry.set_text(self.changes['serverip'])
        self.port_entry.set_text(str(self.changes['serverport']))
        self.backup_location.set_current_folder(self.changes['BackupLocation'])
        self.db_backup_checkbutton.set_active(int(self.changes['backup_db']))
        self.storage_location.set_current_folder(self.changes['serverstoragedir'])
        if self.changes['backup_schedule'] == 'daily':
          self.schedule_daily_button.set_active(True)
        elif self.changes['backup_schedule'] == 'weekly':
          self.schedule_weekly_button.set_active(True)
          wday = int(self.changes['backup_schedule_weekday'])
          self.weekly_combobox.set_active(wday)
        elif self.changes['backup_schedule'] == 'monthly':
          self.schedule_monthly_button.set_active(True)
          md = int(self.changes['backup_schedule_day'])
          mday = md-1
          self.monthly_combobox.set_active(mday)
        elif self.changes['backup_schedule'] == 'disabled':
          self.schedule_disabled_button.set_active(True)
        if not self.changes['backup_schedule'] == "disabled":
          shour = self.changes['backup_schedule_hour']
          self.schedule_hour_combobox.set_active(int(shour))
          smin = int(self.changes['backup_schedule_minute'])/5
          self.schedule_minute_combobox.set_active(int(smin))
        self.schedule_setter()
       
    def schedule_setter(self,widget=None,data=None):
        self.hbox_schedule.hide()
        self.hbox_schedule_weekly.hide()
        self.hbox_schedule_monthly.hide()
        self.monthly_text_label.hide()
        self.monthly_the_label.hide()
        self.schedule_label_sent.hide()
        self.hbox_time.show()
        texthour = self.schedule_hour_combobox.get_active_text()
        textminute = self.schedule_minute_combobox.get_active_text()
        if self.schedule_weekly_button.get_active():
          self.schedule_label_sent.show()
          self.hbox_schedule.show()
          self.hbox_schedule_weekly.show()
          self.schedule_label_sent.set_text("Scheduled backup will be set to run weekly on "+str(self.weekly_combobox.get_active_text())+" at "+texthour+":"+textminute)
        elif self.schedule_monthly_button.get_active():
          self.schedule_label_sent.show()
          self.monthly_text_label.show()
          self.monthly_the_label.show()
          self.hbox_schedule.show()
          self.hbox_schedule_monthly.show()
          self.schedule_label_sent.set_text("Scheduled backup will be set to run monthly on the "+str(self.monthly_combobox.get_active_text())+" day at "+texthour+":"+textminute)
        elif self.schedule_daily_button.get_active():
          self.schedule_label_sent.show()
          self.schedule_label_sent.set_text("Scheduled backup will be set to run daily at "+texthour+":"+textminute)
        if self.schedule_disabled_button.get_active():
          self.hbox_time.hide()

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        changes = False
        print("*********************")
        if not self.changes['serverip'] == self.serverip_entry.get_text():
          changes = True
          print("serverip")
        if not self.changes['serverport'] == self.port_entry.get_text():
          changes = True
          print("serverport")
        if not self.changes['managed'] == str(self.managed_radiobutton.get_active()) and not self.serverip_entry.get_text() == "127.0.0.1":
          changes = True
          print("managed")
        if not self.changes['managed'] == str(self.managed_radiobutton.get_active()) and self.serverip_entry.get_text() == "127.0.0.1":
          self._markReconfigureUser('Error: Manager IP Address cannot be 127.0.0.1 in managed mode', False)
        if not self.changes['BackupLocation'] == self.backup_location.get_current_folder():
          changes = True
          print("location")
        if not self.changes['serverstoragedir'] == self.storage_location.get_current_folder():
          changes = True
          print("serverstoragedir")
        if self.db_backup_checkbutton.get_active() == True:
          self.db_checkbox = "1"
        else:
          self.db_checkbox = "0"
        if not self.changes['backup_db'] == self.db_checkbox:
          changes = True
          print("db")
        if self.schedule_daily_button.get_active():
          self.schedule = "daily"
        elif self.schedule_weekly_button.get_active():
          self.schedule = "weekly"
        elif self.schedule_monthly_button.get_active():
          self.schedule = "monthly"
        elif self.schedule_disabled_button.get_active():
          self.schedule = "disabled"
        if not self.changes['backup_schedule'] == self.schedule:
          changes = True
          print("schedule")
        if self.schedule == "weekly":
          if not self.changes['backup_schedule_weekday'] == self.weekly_combobox.get_active():
            changes = True
            print("weekday")
        elif self.schedule == "monthly":
          if not self.changes['backup_schedule_day'] == self.monthly_combobox.get_active_text():
            changes = True
            print("day")
        if not self.schedule == "disabled":        
          if not self.changes['backup_schedule_hour'] == self.schedule_hour_combobox.get_active_text():
            print("hour")
            changes = True
          if not self.changes['backup_schedule_minute'] == self.schedule_minute_combobox.get_active_text():
            print("minute")
            changes = True
        if changes == True:
          self._markReconfigureUser('Update Configuration', True)

    def hide_stuff(self):
        self.backup_u1_button.hide()
        self.backup_dropbox_button.hide()
        self.hbox_backup_to.hide()
        self.alignment_preconfigured_label.hide()

    def on_managed_toggled(self,widget=None,data=None):
        if self.managed_radiobutton.get_active():
          self.vbox_ip.show()
          self.vbox_backup_location.show()
          self.vbox_set_schedule.show()
        else:
          self.vbox_ip.hide()
          self.vbox_backup_location.hide()
          self.vbox_set_schedule.hide()

    def PickUnusedPort(self):
      s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      s.bind(('localhost', 0))
      addr, port = s.getsockname()
      s.close()
      return port

    #
    # Callbacks
    #
    def on_refresh_button_clicked(self,widget=None,data=None):
        """Shows an example of how a callback can do stuff"""
        self.update_status_window()

    def on_clear_button_clicked(self,widget,data=None):
        """Shows an example of how a callback can do stuff"""
        f = open("/var/lib/mythtv/bare/client.db", 'w')
        f.write("HOST\tLASTBACKUP\tLASTSEEN\tSTATUS")
        f.close()
        self.update_status_window()

    def update_status_thread(self):
#        while True:
        self.update_status_window()
        print("update")
        time.sleep(1)
        return True

    def update_status_window(self):
        f = open("/var/lib/mythtv/bare/client.db")
        host_lines = ""
        lastbackup_lines = ""
        lastseen_lines = ""
        status_lines = ""
        for l in f:
          try:
            line = l.strip('\n')
            host, lastbackup, lastseen, status = line.split('\t')
            if not host == 'HOST':
              host_lines += host+'\n'
              lastbackup_lines += lastbackup+'\n'
              lastseen_lines += lastseen+'\n'
              status_lines += status+'\n'
          except:
            print("Invalid line in client.db")
        f.close
        self.status_host_buffer.set_text(host_lines)
        self.status_lastbackup_buffer.set_text(lastbackup_lines)
        self.status_lastseen_buffer.set_text(lastseen_lines)
        self.status_status_buffer.set_text(status_lines)

    #
    # Process selected activities
    #

    def user_scripted_changes(self,reconfigure):
        """Local changes that can be performed by the user account.
           This function will be ran by the frontend"""
        self.emit_progress("Starting up", 20)
        time.sleep(2)
        for item in reconfigure:
            if item == "Update Configuration":
              newconfig = configparser.ConfigParser()
              self.emit_progress("Reading configuration file", 30)
              try:
                newconfig.read(CONFIGFILE)
                revision=int(newconfig.get("General", "revision"))
                revision += 1
              except:
                revision = 0
                newconfig.add_section("General")
                newconfig.add_section("Backup")
              self.emit_progress("Setting general settings", 40)
              newconfig.set("General", "serverip", self.serverip_entry.get_text())
              newconfig.set("General", "serverport", self.port_entry.get_text())
              newconfig.set("General", "managed", self.managed_radiobutton.get_active())
              newconfig.set("General", "revision", revision)
              newconfig.set("General", "serverstoragedir", self.storage_location.get_current_folder())
              self.emit_progress("Setting backup settings", 50)
              newconfig.set("Backup", "storagedir", self.backup_location.get_current_folder())
              newconfig.set("Backup", "db", self.db_checkbox)
              newconfig.set("Backup", "schedule", self.schedule)
              if self.schedule == "weekly":
                newconfig.set("Backup", "weekday", self.weekly_combobox.get_active())
                newconfig.set("Backup", "day", "*")
              elif self.schedule == "monthly":
                newconfig.set("Backup", "weekday", "*")
                newconfig.set("Backup", "day", self.monthly_combobox.get_active_text())
              elif self.schedule == "monthly":
                newconfig.set("Backup", "weekday", "*")
                newconfig.set("Backup", "day", "*")
              else:
                newconfig.set("Backup", "weekday", "*")
                newconfig.set("Backup", "day", "*")
              self.emit_progress("Setting schedule", 60)
              if not self.schedule == "disabled":        
                newconfig.set("Backup", "hour", self.schedule_hour_combobox.get_active_text())
                newconfig.set("Backup", "minute", self.schedule_minute_combobox.get_active_text())
              newconfig.set("General", "checksum", "XXXXXXXXXX")
              with open(CONFIGFILE, 'w') as conffile:
                newconfig.write(conffile)
              self.emit_progress("Generating hash", 70)
              h = hashlib.sha1()
              f = open(CONFIGFILE, "rb")
              h.update(f.read())
              hsh = h.hexdigest()
              f.close()
              hashconfig = configparser.ConfigParser()
              hashconfig.read(CONFIGFILE)
              hashconfig.set("General", "checksum", hsh)
              self.emit_progress("Saving configuration file", 80)
              with open(CONFIGFILE, 'w') as conffile:
                hashconfig.write(conffile)
        self.emit_progress("Finished setting configuration", 90)
        time.sleep(2)



#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################
import os
import tarfile
import time
import shutil
import subprocess
import platform
import configparser
import logging
import datetime

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-client.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)

ACTOR="<mythbackup> "

class Backup():

    def backup_job(self, BackupDB=True, Location="/var/lib/mythtv/bare-client/"):
        """Backup all the files"""
        logger.info(ACTOR+'Starting backup job')
        ## Check backup directory exists
        if Location == "U1" or Location == "DROPBOX":
            logger.info(ACTOR+'Configured to backup to: '+str(Location))
        elif not os.path.exists(Location):
            os.makedirs(Location)
            logger.info(ACTOR+'Creating backup directory at '+str(Location))
        ## Array of files to Backup
        BACKUPFILES = [
        "/etc/mythtv/config.xml",
        "/etc/mythtv/mysql.txt",
        "/etc/lirc/lircd.conf",
        "/etc/lirc/hardware.conf",
        "/etc/hostname",
        "/etc/hosts",
        ]
        USER=os.getenv("HOME")
        logger.info(ACTOR+'Setting files to backup')
        BACKUPFILES.append(USER+"/.lirc")
        ## Set up backup location
        TMPDIR="/tmp/mythbuntu-bare"
        logger.info(ACTOR+'Creating temp directory at '+str(TMPDIR))
        if not os.path.exists(TMPDIR):
            os.makedirs(TMPDIR)
        BACKUPFILES.append(TMPDIR)
        ## Set status file
        STATUSFILE="/tmp/mythbuntu-bare-status"
        logger.info(ACTOR+'Setting status file to '+str(STATUSFILE))
        if os.path.isfile(STATUSFILE):
            os.remove(STATUSFILE)
        timestamp=time.strftime("%Y%m%d-%H%M", time.localtime())
        ## Backup database only if told to do so
        DBB=""
        MBE=""
        logger.info(ACTOR+'Detecting Master Backend')
        if os.path.isdir("/usr/share/doc/mythtv-backend-master"):
            logger.info(ACTOR+'Master Backend status is True')
            MBE="_MBE"
            DBB="_NODB"
            if BackupDB == '1' or BackupDB == True or BackupDB == "True":
                logger.info(ACTOR+'Backing up database')
                #Run DB Backup
                os.system("echo '20\nBacking up database (this could take a few minutes)'>"+STATUSFILE)
                backupscript="/usr/share/mythtv/mythconverg_backup.pl"
                pipe = subprocess.Popen([backupscript, '--directory', TMPDIR], stdout=subprocess.PIPE).communicate()
                DBB="_YESDB"
            else:
                logger.info(ACTOR+'Database set to not be backed up')
        else:
            logger.info(ACTOR+'Master Backend status is False')
        ## Setup for final tar file
        hostname = platform.node()
        logger.info(ACTOR+'Hostname is '+str(hostname))
#        tarfilename=Location+'/mythbuntu-system-backup_'+hostname+'_'+timestamp+MBE+DBB+'.tar.gz'
        tarfilename='mythbuntu-system-backup_'+hostname+'_'+timestamp+MBE+DBB+'.tar.gz'
        logger.info(ACTOR+'Tarfile is '+str(tarfilename))
        TF=tarfile.open("/tmp/"+tarfilename,mode='w:gz')
        os.system("echo '90\nZipping up files'>"+STATUSFILE)
        ## Add files to tarball
        logger.info(ACTOR+'Backing up files')
        for item in BACKUPFILES:
            if os.path.exists(item):
                TF.add(item)
        TF.close()
        ## Check if managed and push file to server if necessary
        if os.path.isfile("/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf"):
            config = configparser.ConfigParser()
            config.read("/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf")
            managed = config.get("General", "managed")
        else:
            managed = False
        ## Set last backup time
        lastbackupname="lastbackuptime-"+hostname+".log"
        lbt = open("/tmp/"+lastbackupname, 'w')
        lbt.write(str(datetime.datetime.now()))
        lbt.close()
        ## Copy files to backup location
        logger.info(ACTOR+'Copying backup file to: '+Location)
        if Location == "U1":
            import u1files
            u1 = u1files.U1Files()
            u1.login()
            u1.put("/tmp/"+tarfilename, "mythbuntu-bare/"+tarfilename)
            u1.put("/tmp/"+lastbackupname, "mythbuntu-bare/"+lastbackupname)
        elif Location == "DROPBOX":
            ## TODO
            pass
        else:
            shutil.move("/tmp/"+tarfilename, Location+"/"+tarfilename)
            shutil.move("/tmp/"+lastbackupname, Location+"/"+lastbackupname)
#        if managed:
#            logger.info(ACTOR+'Pushing backup file to server')
#            ServerIP = config.get("General", "serverip")
#            ServerPort = config.get("General", "serverport")
#            pipe = subprocess.Popen(["/usr/share/mythbuntu-bare/bareclient/bare/push_file.py", 'http://'+ServerIP+":"+ServerPort+"/cgi-bin/save_file.py", tarfilename], stdout=subprocess.PIPE).communicate()
#        else:
#            logger.info(ACTOR+'Machine unmanaged')

        ## Remove tmp Backup directory and status file
        logger.info(ACTOR+'Removing temp files')
        if os.path.exists(TMPDIR):
            shutil.rmtree(TMPDIR)
        if os.path.isfile(STATUSFILE):
            os.remove(STATUSFILE)
        logger.info(ACTOR+'Backup job finished')

if __name__ == "__main__":
    ACTOR="<backup-task> "

    CONFIGFILENAME = "mythbuntu-bare-client.conf"
    CONFIGPATH = "/var/lib/mythtv/bare-client/"
    CONFIGFILE = CONFIGPATH+CONFIGFILENAME

    ## Read config file
    config = configparser.ConfigParser()
    config.read(CONFIGFILE)

    logger.info(ACTOR+'Initializing backup settings')

    ## Check if managed and if so, then update
    managed = config.get("General", "managed")
    logger.info(ACTOR+'Checking if managed: '+str(managed))
    if managed:
        pipe = subprocess.Popen(["/usr/share/mythbuntu-bare/bareclient/mythbareupdate.py"], stdout=subprocess.PIPE).communicate()

    ## Reread config file in case it was updated
    config.read(CONFIGFILE)
    DB = config.get("Backup", "db")
    logger.info(ACTOR+'Setting database backup to '+str(DB))
    storagedir = config.get("Backup", "storagedir")
    logger.info(ACTOR+'Setting backup storage directory to '+str(storagedir))

    ## Run backup job
    logger.info(ACTOR+'Starting backup job')
    Backup().backup_job(DB, storagedir)



#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
# «Bare Server» - The webserver for Mythbuntu Bare console
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from http.server import CGIHTTPRequestHandler
from http.server import HTTPServer
import sys
import os
import logging
import signal

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-server.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

ACTOR="<Webserver> "

class BareWebserver():

  def start_webserver(self, PORT):
    cgi_directories = ["/cgi-bin"]
    server_address=('',PORT)
    os.chdir("/var/lib/mythtv/bare/")
    httpd = HTTPServer(server_address, CGIHTTPRequestHandler)
    signal.signal(signal.SIGTERM, self.signal_handler)
    httpd.serve_forever()

  def signal_handler(self, signal, frame):
    httpd.server_close()
    logger.info(ACTOR+'Stopping BareServer')
    sys.exit(0)

if __name__ == "__main__":
  import configparser
  newconfig = configparser.ConfigParser()
  CONFIGFILE = "/var/lib/mythtv/bare/mythbuntu-bare.conf"
  logger.info(ACTOR+'Starting BareServer')
  try:
    newconfig.read(CONFIGFILE)
    serverport=int(newconfig.get("General", "serverport"))
    logger.info(ACTOR+'Setting communication port to: '+str(serverport))
  except:
    logger.error(ACTOR+'Cannot find communication port')
    sys.exit(1)
  try:
    BareWebserver().start_webserver(serverport)
    logger.info(ACTOR+'Bare Server Started')
  except IOError as xxx_todo_changeme: 
    (value,message) = xxx_todo_changeme.args 
    logger.error(ACTOR+'%s: %s' % (value, message))
    sys.exit(1)




#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import logging

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-client.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

ACTOR="<updater> "

class CronUpdater():

    def generate_cron_schedule(self):
      logger.info(ACTOR+'Cron update job started')
      import configparser
      config = configparser.ConfigParser()
      logger.info(ACTOR+'Reading conf file')
      config.read("/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf")
      managed = config.get("General", "managed")
      schedule = config.get("Backup", "schedule")
      day = config.get("Backup", "day")
      hour = config.get("Backup", "hour")
      minute = config.get("Backup", "minute")
      weekday = config.get("Backup", "weekday")
      logger.info(ACTOR+'Detected Managed:'+str(managed)+' Schedule:'+str(schedule)+' Day:'+str(day)+' Hour:'+str(hour)+' Minute:'+str(minute)+' Weekday'+str(weekday))
      f = open("/etc/cron.d/mythbuntu-bare","w")
      if not schedule == "disabled":
          logger.info(ACTOR+'Writing backup schedule cron job')
          f.write(minute+" "+hour+" "+day+" "+"*"+" "+weekday+" root /usr/share/mythbuntu-bare/bareclient/mythbackup.py\n")
      if managed:
          logger.info(ACTOR+'Writing update task cron job')
          f.write("41 * * * * root /usr/share/mythbuntu-bare/bareclient/mythbareupdate.py\n")
      f.close()
      logger.info(ACTOR+'Cron update job finished')




#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import pycurl
import sys
import logging

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-client.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

ACTOR="<push_file> "

# Check commandline arguments
if len(sys.argv) < 3:
    print("Usage: %s <url> <file to upload>" % sys.argv[0])
    raise SystemExit
url = sys.argv[1]
filename = sys.argv[2]
 
c = pycurl.Curl()
 
values = [
     ("filename", (pycurl.FORM_FILE, filename))
]

logger.info(ACTOR+'Pushing '+str(filename)+'to'+str(url))

c.setopt(c.URL, url)
c.setopt(c.HTTPPOST, values)
c.perform()
logger.info(ACTOR+'Return code: '+str(c.getinfo(pycurl.HTTP_CODE))+" from "+str(c.getinfo(pycurl.EFFECTIVE_URL)))
c.close()



#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import configparser
import os
import shutil
import hashlib
import platform
import logging

CONFIGFILENAME = "mythbuntu-bare-client.conf"
CONFIGPATH = "/var/lib/mythtv/bare-client/"

TMPCONFIGFILENAME="mythbuntu-bare.conf"

CONFIGFILE = CONFIGPATH+CONFIGFILENAME
TMPCONFIGFILE = "/tmp/"+TMPCONFIGFILENAME

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-client.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

ACTOR="<mythbareupdate> "

class Update():

    def update_job(self, ServerIP=None, ServerPort=None):
        logger.info(ACTOR+'Update job started')
        logger.info(ACTOR+'Gathering configuration')
        if os.path.isfile(CONFIGFILE):
            NEW = False
            config = configparser.ConfigParser()
            config.read(CONFIGFILE)
            self.Revision = config.get("General", "Revision")
            if config.getboolean("General", "managed"):
                ServerIP = config.get("General", "serverip")
                ServerPort = config.get("General", "serverport")
                ServerURL = "http://"+ServerIP+":"+ServerPort+"/"
            else:
                ServerURL = "NONE"
            logger.info(ACTOR+'Server is located at '+str(ServerURL))
            Location = config.get("Backup", "storagedir")
            logger.info(ACTOR+'Backup location is set to '+str(Location))
        else:
            NEW = True
            self.Revision = 0
            ServerURL = "http://"+ServerIP+":"+ServerPort+"/"
            logger.info(ACTOR+'Server not set in conf file. Using passed in settings '+str(ServerURL))
            Location = "/var/lib/mythtv/bare-client/"
            logger.info(ACTOR+'Backup location not set in conf file. Setting to '+str(Location))

        ## Get last backup time
        if os.path.isfile(Location+"/lastbackuptime.log"):
            lbtf = open(Location+"/lastbackuptime.log")
            lbt = lbtf.readline()
            lbtf.close()
        else:
            logger.info(ACTOR+'lastbackuptime.log does not exist')
            lbt = "NA"

        ## Write check in file
        CHECKINCREATED=False
        try:
            cif = open("/tmp/"+HOSTNAME+"-bare-checkinfile.barelog", 'w')
            cif.write(HOSTNAME+"\t"+lbt+"\tNA\tNA")
            cif.close()
            HOSTNAME=str(platform.node())
            CHECKINCREATED=True
        except:
            logger.debug(ACTOR+"Check in file creation failed")

        ## Push check in file to server
        if CHECKINCREATED:
            logger.info(ACTOR+"Pushing check in file to server")
            pipe = subprocess.Popen(["/usr/share/mythbuntu-bare/bareclient/bare/push_file.py", "http://"+ServerIP+":"+ServerPort+"/cgi-bin/save_file.py", "/tmp/"+HOSTNAME+"-bare-checkinfile.barelog"], stdout=subprocess.PIPE).communicate()
            os.remove("/tmp/"+HOSTNAME+"bare-checkinfile.barelog")
        else:
            logger.info(ACTOR+"Check in file wasn't newly created, skipping file upload")

        ## Check tmp dir for update file
        if os.path.isfile(TMPCONFIGFILE):
            logger.info(ACTOR+'Processing file '+str(TMPCONFIGFILE))
            NEW = False
            self.process_updates()
            os.remove(TMPCONFIGFILE)

        ## Download update file from server
        download_successful=False
        try:
            logger.info(ACTOR+'Checking server for config file')
            self.check_server(TMPCONFIGFILENAME,ServerURL)
            download_successful=True
            logger.info(ACTOR+'Configuration file was downloaded')
        except:
            logger.debug(ACTOR+'Could not download file from server')

        ## If files downloaded from server, get revision and process them
        if download_successful:
            if NEW == False:
                config = configparser.ConfigParser()
                config.read(CONFIGFILE)
                logger.info(ACTOR+'Reading conf file '+str(CONFIGFILE))
                Revision = config.get("General", "Revision")
            else:
                NEW = False
                Revision = 0
            self.process_updates()

        ## Remove temp files
        logger.info(ACTOR+'Removing temp files')
        if os.path.isfile(TMPCONFIGFILE):
            os.remove(TMPCONFIGFILE)
        logger.info(ACTOR+'Update job finished')

    def check_server(self,file_name,base_url):
        from urllib.request import Request, urlopen
        from urllib.error import URLError, HTTPError
        #create the url and the request
        url = base_url + file_name
        req = Request(url)
        logger.info(ACTOR+'Downloading conf file from '+str(url))
        # Open the url
        f = urlopen(req)
        # Open our local file for writing
        local_file = open(TMPCONFIGFILE, "w")
        #Write to our local file
        local_file.write(f.read())
        local_file.close()

    def process_updates(self):
        ## Get revision
        tempconfig = configparser.ConfigParser()
        tempconfig.read(TMPCONFIGFILE)
        logger.info(ACTOR+'Reading conf file '+str(TMPCONFIGFILE))
        tempRevision = tempconfig.get("General", "Revision")
        logger.info(ACTOR+'Configuration file revision is '+str(tempRevision))
        logger.info(ACTOR+'Current revision is '+str(self.Revision))
        ## If revision is newer, process. 
        if int(self.Revision) < int(tempRevision):
            logger.info(ACTOR+'Downloaded file is new')
            ## Calculate checksum
            tempChecksum = tempconfig.get("General", "checksum")
            tempconfig.set("General", "checksum", "XXXXXXXXXX")
            logger.info(ACTOR+'Verifying file integrity')
            with open("/tmp/mythbuntu-bare.tmp", 'w') as conffile:
                tempconfig.write(conffile)
            h = hashlib.sha1()
            f = open("/tmp/mythbuntu-bare.tmp", "rb")
            h.update(f.read())
            hsh = h.hexdigest()
            f.close()
            os.remove("/tmp/mythbuntu-bare.tmp")
            if tempChecksum == hsh:
                logger.info(ACTOR+'Importing new config file')
                ## If checksums match, copy file into dir
                shutil.copyfile(TMPCONFIGFILE, CONFIGFILE)
            else:
                logger.debug(ACTOR+'Error Checksum failed. '+tempChecksum+' '+hsh)
        else:
            logger.info(ACTOR+'Downloaded config file is not new')

if __name__ == "__main__":
    Update().update_job() 



#!/usr/bin/python
## -*- coding: utf-8 -*-
#
# Copyright © 2006 Clodoaldo Pinto Neto cpn@codepoint.net
#
# All code, and only the code, in webpython.codepoint.net is free software; 
# you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import cgi, os
import cgitb; cgitb.enable()
import sys

UPLOADDIR='files/'

form = cgi.FieldStorage()

# Get filename here.
fileitem = form['filename']

# Test if the file was uploaded
if fileitem.filename:
   # strip leading path from file name to avoid 
   # directory traversal attacks
   fn = os.path.basename(fileitem.filename)
   ## Check filename matches expected values
   print """\Content-Type: text/html\n"""
   if os.path.isfile(UPLOADDIR + fn):
     message = "ERROR: File Exists"
     EXIT=1
   elif not fn.endswith(".tar.gz") or not fn.endswith(".barelog"):
     message = "ERROR: File is invalid"
     EXIT=2
   else:
     open(UPLOADDIR + fn, 'wb').write(fileitem.file.read())
     message = fn + ' was uploaded successfully'
     EXIT=0
else:
   message = 'No file was uploaded'
   
#print """\Content-Type: text/html\n"""
print """%s""" % (message,)
sys.exit(EXIT)






## -*- coding: utf-8 -*-
#
# «mythbuntu-bare» - A Plugin to assist in backing up and restoring the mythtv DB
#
# Copyright (C) 2010, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
from gi.repository import Gtk
import time
import os
import tarfile
import shutil
import subprocess
from threading import Thread
import sys
import configparser
import hashlib
import traceback
sys.path.append("/usr/share/mythbuntu-bare/bareclient")
from mythbackup import Backup
from mythrestore import Restore


class MythbuntuBaRePlugin(MCCPlugin):
    """A Plugin to assist in backing up and restoring the mythtv DB"""
    #
    #Load GUI & Calculate Changes
    #
    
    CONFIGFILE = "/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf"

    #USER=os.getenv("HOME")  ## This doesn't work when passed to root changes
    USER=os.path.expanduser('~')

    BACKUPFILES = [
    "/etc/mythtv/config.xml",
    "/etc/mythtv/mysql.txt",
    "/etc/lirc/lircd.conf",
    "/etc/lirc/hardware.conf",
    "/etc/udev/rules.d"
    ]

    BACKUPFILES.append(USER+"/.lirc")

    BACKUP_LOCATION = USER
    BACKUP_DB=False
    restore_file = '/tmp'

    IsTarFile = False
    ValidNewConf = False

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Backup and Restore"
        information["icon"] = "revert"
        information["ui"] = "tab_mythbuntu-bare"
        MCCPlugin.__init__(self,information)
        import configparser
        self.config = configparser.ConfigParser()

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        import os
        self.changes = {}
        try:
            self.config.read(self.CONFIGFILE)
            self.changes['serverip'] = self.config.get("General", "serverip")
            self.changes['managed'] = self.config.get("General", "managed")
            if str.lower(self.changes['managed']) == "true":
                self.managed_label.set_text("This machine is current in managed mode")
                self.radio_convert.set_sensitive(False)
                self.backup_schedule.set_sensitive(False)
            self.changes['BackupLocation'] = self.config.get("Backup", "storagedir")
            self.changes['backup_db'] = self.config.get("Backup", "db")
            self.changes['backup_schedule'] = self.config.get("Backup", "schedule")
            self.changes['backup_schedule_day'] = self.config.get("Backup", "day")
            self.changes['backup_schedule_hour'] = self.config.get("Backup", "hour")
            self.changes['backup_schedule_minute'] = self.config.get("Backup", "minute")
            self.changes['backup_schedule_weekday'] = self.config.get("Backup", "weekday")
            self.db_backup_checkbutton.set_active(self.changes['backup_db'])
        except:
            print("WARNING: Failed to set from config file. Setting defaults")
            traceback.print_exc(file=sys.stdout)
            self.changes['serverip'] = "127.0.0.1"
            self.changes['managed'] = "False"
            self.changes['BackupLocation'] = "/var/lib/mythtv/bare/"
            self.changes['backup_db'] = "False"
            self.changes['backup_schedule'] = "disabled"
            self.changes['backup_schedule_day'] = "*"
            self.changes['backup_schedule_hour'] = "0"
            self.changes['backup_schedule_minute'] = "00"
            self.changes['backup_schedule_weekday'] = "*"
        if not self.query_installed('mythtv-backend-master'):
            self.backup_db_label.set_text("Database can only be backed up from master backend")
            self.db_backup_checkbutton.set_sensitive(False)
            self.db_backup_checkbutton.set_active(False)

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.button_box.hide()
        self.hide_tabs()
        self.vbox_start.show()
#        self.vbox_backup.hide()
        self.vbox_restore.hide()
        self.vbox_choices.hide()
        self.radio_backup.set_active(False)
        ## Set radio buttons to default (off)
        self.radio_backup_default_off.set_active(True)
        self.radio_backup_default_off.hide()
        self.radio_schedule_default_off.set_active(True)
        self.radio_schedule_default_off.hide()
        self.backup_schedule_off.set_active(True)
        self.backup_schedule_off.hide()
        self.radio_restore.set_active(False)
        self.backup_now.set_active(False)
        self.backup_dir_button.set_active(False)
#        self.on_radio_backup_toggled(None)
        self.on_backup_dir_button_toggled(None)
#        self.on_backup_schedule_toggled(None)
        
        shour = self.changes['backup_schedule_hour']
        self.schedule_hour_combobox.set_active(int(shour))
        smin = int(self.changes['backup_schedule_minute'])/5
        self.schedule_minute_combobox.set_active(int(smin))
        if self.changes['backup_schedule'] == "disabled":
            self.scheduled_label.set_text("Disabled")
            self.schedule_disabled_button.set_active(True)
        elif self.changes['backup_schedule'] == "daily":
           self.schedule_daily_button.set_active(True)
        elif self.changes['backup_schedule'] == "weekly":
            self.schedule_weekly_button.set_active(True)
            wday = int(self.changes['backup_schedule_weekday'])
            self.weekly_combobox.set_active(wday)
            self.hbox_schedule_weekly.show()
        elif self.changes['backup_schedule'] == "monthly":
            self.schedule_monthly_button.set_active(True)
            md = int(self.changes['backup_schedule_day'])
            mday = md-1
            self.monthly_combobox.set_active(mday)
            self.hbox_schedule_monthly.show()
        if self.changes['backup_schedule_day'] == "*":
            self.monthly_combobox.set_active(0)
        if self.changes['backup_schedule_weekday'] == "*":
            self.weekly_combobox.set_active(0)
        self.restore_label_isfile.hide()
        self.managed_label_isfile.hide()
        ## Filter only tar.gz files for restore
        filter = Gtk.FileFilter()
        filter.set_name("Backup Files")
        filter.add_pattern("*.tar.gz")
        self.restore_file.add_filter(filter)
        self.restore_file.unselect_all()
        conffilter = Gtk.FileFilter()
        conffilter.set_name("Config Files")
        conffilter.add_pattern("*.conf")
        self.managedfilechooser.add_filter(conffilter)
        self.managedfilechooser.unselect_all()
        self.backup_location.unselect_all()
        ## Disable non-implemented items
        self.backup_dropbox_button.hide()
        self.backup_u1_button.hide()
        self.radio_convert.hide()
        self.managed_label.hide()

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        if self.radio_backup.get_active():
            if self.backup_dir_button.get_active():
                self.BACKUP_LOCATION=self.backup_location.get_filename()
            self.BACKUP_DB = self.db_backup_checkbutton.get_active()
            if self.backup_now.get_active():
                self._markReconfigureUser('BackupNow',self.BACKUP_LOCATION)
            elif self.backup_schedule.get_active():
                #TODO
                if self.schedule_disabled_button.get_active():
                    self._markReconfigureRoot('BackupSchedule', False)
                else:
                    self._markReconfigureRoot('BackupSchedule', True)
                    self._markReconfigureRoot('BackupHour', self.schedule_hour_combobox.get_active_text())
                    self._markReconfigureRoot('BackupMinute', self.schedule_minute_combobox.get_active_text())
                    self._markReconfigureRoot('BackupLocation', self.BACKUP_LOCATION)
                    self._markReconfigureRoot('Save Backup Location', self.BACKUP_LOCATION)
                    self._markReconfigureRoot('BackupDatabase', str(self.db_backup_checkbutton.get_active()))
                    if self.schedule_daily_button.get_active():
                       self._markReconfigureRoot('BackupDaily', True)
                    elif self.schedule_weekly_button.get_active():
                       self._markReconfigureRoot('BackupWeekly', str(self.weekly_combobox.get_active()))
                    elif self.schedule_monthly_button.get_active():
                        self._markReconfigureRoot('BackupMonthly', str(self.monthly_combobox.get_active_text()))
        if self.radio_restore.get_active():
            self._markReconfigureRoot('restore',self.restore_file.get_filename())
        if self.restore_db.get_active():
            self._markReconfigureRoot('restore_db',True)
        if self.radio_convert.get_active():
            self._markReconfigureRoot('convert',self.managedfilechooser.get_filename())

    #
    # Callbacks
    #

    def on_restore_file_selection_changed(self,widget,data=None):
        if self.restore_file.get_filename():
            tarfilename=self.restore_file.get_filename()
            if tarfile.is_tarfile(tarfilename):
                TF=tarfile.open(tarfilename,mode='r:gz')
                for line in TF.getnames():
                    if line.endswith('sql.gz'):
                        self.RESTORE_DB_EXISTS=True
                        break
                    else:
                        self.RESTORE_DB_EXISTS=False
                        self.restore_db.set_active(False)
                TF.close()
                self.IsTarFile=True
                self.restore_label_isfile.hide()
            else:
                self.restore_label_isfile.show()

    def on_managedfilechooser_selection_changed(self,widget,data=None):
        if self.managedfilechooser.get_filename():
            newconfigfilename=self.managedfilechooser.get_filename()
            newconfig = configparser.ConfigParser()
            newconfig.read(newconfigfilename)
            self.changes['checksum'] = newconfig.get("General", "checksum")
            newconfig.set("General", "checksum", "XXXXXXXXXX")
            with open("/tmp/mythbuntu-bare.tmp", 'w') as conffile:
                newconfig.write(conffile)
            h = hashlib.sha1()
            f = open("/tmp/mythbuntu-bare.tmp", "rb")
            h.update(f.read())
            hsh = h.hexdigest()
            f.close()
            if hsh == self.changes['checksum']:
                try:
                    self.changes['serverip'] = newconfig.get("General", "serverip")
                    self.changes['serverport'] = newconfig.get("General", "serverport")
                    self.changes['managed'] = newconfig.get("General", "managed")
                    self.changes['BackupLocation'] = newconfig.get("Backup", "storagedir")
                    self.changes['backup_db'] = newconfig.get("Backup", "db")
                    self.changes['backup_schedule_day'] = newconfig.get("Backup", "day")
                    self.changes['backup_schedule_hour'] = newconfig.get("Backup", "hour")
                    self.changes['backup_schedule_minute'] = newconfig.get("Backup", "minute")
                    self.changes['backup_schedule_weekday'] = newconfig.get("Backup", "weekday")
                    self.changes['backup_schedule'] = newconfig.get("Backup", "schedule")
                    self.ValidNewConf = True
                    self.managed_label_isfile.hide()
                except:
                    self.ValidNewConf = False
                    self.managed_label_isfile.show()
            else:
                print("Error: Signature failure. Configuration file possibly corrupt.")

    def on_button_started_clicked(self,widget,data=None):
        self.tab_history=[]
        self.CUR_TAB = "vbox_choices"
        self.tab_history.append("vbox_start")
        self.vbox_start.hide()
        self.vbox_choices.show()
        self.button_box.show()
        self.next_button.set_sensitive(True)

    def on_next_button_clicked(self,widget,data=None):
        self.change_tab(self.CUR_TAB,"next")

    def on_back_button_clicked(self,widget,data=None):
        self.hide_tabs()
        if self.CUR_TAB == "vbox_final":
            self.next_button.set_sensitive(True)
        self.CUR_TAB = self.tab_history.pop()
        f = getattr(self, self.CUR_TAB)
        f.show()
        if self.CUR_TAB == "vbox_start":
            self.button_box.hide()

    def change_tab(self,CURRENT,DIRECTION):
        """Monkee around to display the correct wizard page"""
        self.hide_tabs()
        if self.CUR_TAB == "vbox_choices":
            if self.radio_backup_default_off.get_active() == False:
                self.tab_history.append(self.CUR_TAB)
                if self.radio_backup.get_active():
                    self.CUR_TAB = "vbox_backup_choice"
                    self.vbox_backup_choice.show()
                elif self.radio_convert.get_active():
                    self.CUR_TAB = "vbox_managed_convert"
                    self.vbox_managed_convert.show()
                else:
                    self.CUR_TAB = "vbox_restore"
                    self.vbox_restore.show()
            else:
                self.vbox_choices.show()
        elif self.CUR_TAB == "vbox_backup_choice":
            self.tab_history.append(self.CUR_TAB)
            if self.backup_now.get_active():
                self.CUR_TAB = "vbox_backup_location"
                self.vbox_backup_location.show()
                self.backup_type = "now"
            elif self.backup_schedule.get_active():
                self.CUR_TAB = "vbox_set_schedule"
                self.vbox_set_schedule.show()
                self.backup_type = "scheduled"
            else:
                self.vbox_backup_choice.show()
        elif self.CUR_TAB == "vbox_restore":
            if self.IsTarFile == True:
                self.tab_history.append(self.CUR_TAB)
                if self.RESTORE_DB_EXISTS:
                    self.CUR_TAB = "vbox_restore_db"
                    self.vbox_restore_db.show()
                else:
                    self.CUR_TAB = "vbox_final"
                    self.vbox_final.show()
                    self.next_button.set_sensitive(False)
            else:
                self.vbox_restore.show()
        elif self.CUR_TAB == "vbox_restore_db":
            self.tab_history.append(self.CUR_TAB)
            self.CUR_TAB = "vbox_final"
            self.vbox_final.show()
            self.next_button.set_sensitive(False)
        elif self.CUR_TAB == "vbox_backup_location":
            #from u1files import U1Files
            if self.backup_dir_button.get_active():# or (self.BACKUP_LOCATION == "U1" and U1Files().login()):
                self.tab_history.append(self.CUR_TAB)
                self.CUR_TAB = "vbox_final"
                self.vbox_final.show()
                self.next_button.set_sensitive(False)
        elif self.CUR_TAB == "vbox_set_schedule":
            if self.schedule_daily_button.get_active() or (self.schedule_monthly_button.get_active() and not self.monthly_combobox.get_active_text() == None) or (self.schedule_weekly_button.get_active() and not self.weekly_combobox.get_active_text() == None):
                self.tab_history.append(self.CUR_TAB)
                self.CUR_TAB = "vbox_backup_location"
                self.vbox_backup_location.show()
            elif self.schedule_disabled_button.get_active():
                self.tab_history.append(self.CUR_TAB)
                self.CUR_TAB = "vbox_final"
                self.vbox_final.show()
                self.next_button.set_sensitive(False)
        elif self.CUR_TAB == "vbox_managed_convert":
            if self.ValidNewConf == True:
                self.tab_history.append(self.CUR_TAB)
                self.CUR_TAB = "vbox_final"
                self.vbox_final.show()
                self.next_button.set_sensitive(False)
            else:
                self.vbox_managed_convert.show()

    def backup_setter(self,widget,data=None):
        radio_convert = self.radio_convert.get_active()
        radio_restore = self.radio_restore.get_active()
        radio_backup = self.radio_backup.get_active()
        self.set_final_label()

    def schedule_setter(self,widget,data=None):
        self.hbox_schedule.hide()
        self.hbox_schedule_weekly.hide()
        self.hbox_schedule_monthly.hide()
        self.monthly_text_label.hide()
        self.monthly_the_label.hide()
        self.schedule_label_sent.hide()
        texthour = self.schedule_hour_combobox.get_active_text()
        textminute = self.schedule_minute_combobox.get_active_text()
        if self.schedule_weekly_button.get_active():
            self.schedule_label_sent.show()
            self.hbox_schedule.show()
            self.hbox_schedule_weekly.show()
            self.schedule_label_sent.set_text("Scheduled backup will be set to run weekly on "+str(self.weekly_combobox.get_active_text())+" at "+texthour+":"+textminute)
        elif self.schedule_monthly_button.get_active():
            self.schedule_label_sent.show()
            self.monthly_text_label.show()
            self.monthly_the_label.show()
            self.hbox_schedule.show()
            self.hbox_schedule_monthly.show()
            self.schedule_label_sent.set_text("Scheduled backup will be set to run monthly on the "+str(self.monthly_combobox.get_active_text())+" day at "+texthour+":"+textminute)
        elif self.schedule_daily_button.get_active():
            self.schedule_label_sent.show()
            self.schedule_label_sent.set_text("Scheduled backup will be set to run daily at "+texthour+":"+textminute)
        self.set_final_label()

    def set_final_label(self):
        if self.radio_convert.get_active():
            self.job_label.set_text("Please click apply to convert the machine to managed mode")
        if self.radio_restore.get_active():
            self.job_label.set_text("Please click apply to perform the restore procedure")
        if self.radio_backup.get_active():
            if self.backup_schedule.get_active():
                self.job_label.set_text("Please click apply to perform the specified scheduling changes")
            else:
                self.job_label.set_text("Please click apply to perform the backup procedure")

    def on_backup_dir_button_toggled(self,widget,data=None):
        """Prompt user for save location if needed"""
        self.hide_all()
        if self.backup_dir_button.get_active():
            self.backup_location.show()
        elif self.backup_u1_button.get_active():
            self.preconfigured_label.show()
            self.BACKUP_LOCATION = "U1"
        elif self.backup_dropbox_button.get_active():
            self.preconfigured_label.show()
            self.BACKUP_LOCATION = "DROPBOX"
        else:        
            print("Error: Unknown option selected")

#    def on_backup_schedule_toggled(self,widget,data=None):
#        """Show backup schedule options if needed"""
#        if self.backup_schedule.get_active():
#            self.backup_schedule_status.show()
#            if self.changes['BackupLocation'] != "NOWHERE":
#                self.backup_location.set_filename(self.changes['BackupLocation'])
#        else:
#            self.backup_schedule_status.hide()

    def hide_tabs(self):
        self.vbox_start.hide()
        self.vbox_restore.hide()
        self.vbox_restore_db.hide()
        self.vbox_backup_schedule.hide()
        self.vbox_choices.hide()
        self.vbox_backup_choice.hide()
        self.vbox_backup_location.hide()
        self.vbox_final.hide()
        self.vbox_set_schedule.hide()
        self.vbox_managed_convert.hide()

    def hide_all(self):
        """Hide everything before showing what is needed"""
        self.preconfigured_label.hide()
        self.backup_location.hide()
        self.hbox_schedule.hide()
        self.hbox_schedule_weekly.hide()
        self.hbox_schedule_monthly.hide()
        self.monthly_text_label.hide()
        self.monthly_the_label.hide()
        self.schedule_label_sent.hide()

    #
    # Process selected activities
    #

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        self.config.add_section("Backup")
        time.sleep(2)
        RESTORE_SET=False
        RESTORE_DB=False
        STATUSFILE = "/tmp/mythbuntu-bare-status"
        for item in reconfigure:
            if item == "Save Backup Location":
                self.BACKUP_LOCATION = reconfigure[item]
        for item in reconfigure:
            if item == "BackupSchedule":
                self.emit_progress("Processing Backup Schedule", 10)
                if reconfigure[item] == False:
                    self.emit_progress("Removing scheduled job", 50)
                    if os.path.isfile("/etc/cron.d/mythbuntu-bare"):
                        os.remove("/etc/cron.d/mythbuntu-bare")
                elif reconfigure[item] == True:
                    CONFIGFILE = "/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf"
                    self.emit_progress("Saving Backup Settings", 30)
                    import configparser
                    config = configparser.ConfigParser()
                    config.add_section('General')
                    config.add_section('Backup')
                    config.set('General', 'serverip', '127.0.0.1')
                    config.set('General', 'managed', 'false')
                    config.set('General', 'revision', '1')
                    for subitem in reconfigure:
                        if subitem == 'BackupHour':
                            config.set('Backup', 'hour', reconfigure[subitem])
                        elif subitem == 'BackupMinute':
                            config.set('Backup', 'minute', reconfigure[subitem])
                        elif subitem == 'BackupLocation':
                            config.set('Backup', 'storagedir', reconfigure[subitem])
                        elif subitem == 'BackupDatabase':
                            config.set('Backup', 'db', reconfigure[subitem])
                        elif subitem == 'BackupDaily':
                            config.set('Backup', 'schedule', 'daily')
                            config.set('Backup', 'day', '*')
                            config.set('Backup', 'weekday', '*')
                        elif subitem == 'BackupWeekly':
                            config.set('Backup', 'schedule', 'weekly')
                            config.set('Backup', 'day', '*')
                            config.set('Backup', 'weekday', reconfigure[subitem])
                        elif subitem == 'BackupMonthly':
                            config.set('Backup', 'schedule', 'monthly')
                            config.set('Backup', 'day', reconfigure[subitem])
                            config.set('Backup', 'weekday', '*')
                    with open(CONFIGFILE, 'w', encoding='utf-8') as conffile:
                        config.write(conffile)
                    sys.path.append("/usr/share/mythbuntu/bare")
                    from updater import CronUpdater
                    self.emit_progress("Saving Backup Schedule", 10)
                    CronUpdater().generate_cron_schedule()
            elif item == "restore":
                RESTORE_LOCATION=reconfigure[item]
                RESTORE_SET=True
            elif item == "restore_db":
                RESTORE_DB=True
            elif item == "convert":
                PATH=reconfigure[item]
                try:
                    shutil.move(PATH, "/var/lib/mythtv/bare-client/mythbuntu-bare-client.conf")
                    sys.path.append("/usr/share/mythbuntu/bare")
                    from updater import CronUpdater
                    CronUpdater().generate_cron_schedule()
                except:
                    print("Unable to import file")
        if RESTORE_SET == True:
            rj = Thread(target=Restore().restore_job, args=(RESTORE_LOCATION,RESTORE_DB))
            rj.start()
            self.emit_progress("Restoring files", 20)
            time.sleep(2)
            status=20
            while rj.isAlive():
                f = open(STATUSFILE, 'r')
                ANS1 = f.readline().rstrip('\n')
                ANS2 = f.readline().rstrip('\n')
                status = int(ANS1)
                self.emit_progress(ANS2, status)
                f.close()
                while ANS2 == 'Restoring database (this could take a few minutes)':
                    f = open(STATUSFILE, 'r')
                    ANS1 = f.readline().rstrip('\n')
                    ANS2 = f.readline().rstrip('\n')
                    f.close()
                    if status <= 90:
                        status=status+0.1
                        self.emit_progress(ANS2, status)
                        time.sleep(0.1)
                time.sleep(0.1)
            self.emit_progress("Restore Job Complete", 100)
        time.sleep(2)

    def user_scripted_changes(self,reconfigure):
        """Local changes that can be performed by the user account.
           This function will be ran by the frontend"""
        time.sleep(0.5)
        for item in reconfigure:
            if item == "BackupNow":
                STATUSFILE = "/tmp/mythbuntu-bare-status"
                bj = Thread(target=Backup().backup_job, args=(self.BACKUP_DB,self.BACKUP_LOCATION))
                bj.start()
                self.emit_progress("Starting Backup", 10)
                time.sleep(2)
                status=10
                while bj.isAlive():
                    f = open(STATUSFILE, 'r')
                    ANS1 = f.readline().rstrip('\n')
                    ANS2 = f.readline().rstrip('\n')
                    status = int(ANS1)
                    self.emit_progress(ANS2, status)
                    f.close()
                    while ANS2 == 'Backing up database (this could take a few minutes)':
                        f = open(STATUSFILE, 'r')
                        ANS1 = f.readline().rstrip('\n')
                        ANS2 = f.readline().rstrip('\n')
                        f.close()
                        if status <= 85:
                            status=status+0.1
                            self.emit_progress(ANS2, status)
                            time.sleep(0.1)
                    time.sleep(0.1)
        self.emit_progress("Backup Finished", 100)
        time.sleep(2)



#!/usr/bin/python3
## -*- coding: utf-8 -*-
#
# «mythvideo-monitor» - Utility to monitor video storage groups and 
#                         automatically add to mythtv database, scan for
#                         metadata and set group permissions to mythtv with rw 
#
# Copyright (C) 2011, Thomas Mashos, for Mythbuntu
#
# Mythbuntu-bare is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

import pyinotify
import logging
import shutil
import os
import tarfile
import datetime

logger = logging.getLogger('mythbuntu-bare')
hdlr = logging.FileHandler('/var/log/mythbuntu-bare-server.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.WARNING)

INDIR="/var/lib/mythtv/bare/"
ACTOR="<Processor> "

wm = pyinotify.WatchManager()
mask = pyinotify.IN_CLOSE_WRITE
wm.add_watch(INDIR+"files/", mask, rec=True)

class PTmp(pyinotify.ProcessEvent):
  def process_IN_CLOSE_WRITE(self, event):
    self.process_file(event.name)
    logger.info(ACTOR+"Received file: %s " % event.name)

  def process_file(self, FILENAME):
    logger.info(ACTOR+"Processing file: %s " % FILENAME)
    FULLPATH=INDIR+"files/"+FILENAME
    if FILENAME.endswith(".tar.gz"):
      if tarfile.is_tarfile(FULLPATH):
        shutil.move(FULLPATH, INDIR+"backup/")
      else:
        logger.error(ACTOR+"%s is not a valid tarfile" % FILENAME)
    elif FILENAME.endswith(".barelog"):
      f = open(FULLPATH)
      LINE = f.readline()
      HOST, LASTBACKUP, LASTSEEN, STATUS = LINE.strip("\n").split("\t")
      f.close()
      dbtmp = open(INDIR+"client.dbtmp", "w")
      db = open(INDIR+"client.db", "r")
      LINES=db.readlines()
      for l in LINES:
        DHOST, DLASTBACKUP, DLASTSEEN, DSTATUS = l.strip("\n").split("\t")
        if not DHOST == HOST:
          dbtmp.write(DHOST+"\t"+DLASTBACKUP+"\t"+DLASTSEEN+"\t"+DSTATUS+"\n")
      dbtmp.write(HOST+"\t"+LASTBACKUP+"\t"+str(datetime.datetime.now())+"\t"+STATUS+"\n")
      db.close()
      dbtmp.close()
      os.remove(INDIR+"client.db")
      shutil.move(INDIR+"client.dbtmp", INDIR+"client.db")      
      os.remove(FULLPATH)
    else:
      logger.error(ACTOR+"Unknown file: %s " % FILENAME)

notifier = pyinotify.Notifier(wm, PTmp())

while True:
  try:
    notifier.process_events()
    if notifier.check_events():
     notifier.read_events()
  except KeyboardInterrupt:
    notifier.stop()
    break

